# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-12-13 18:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_orghierarchy', '0002_organization_replaced_by'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='organization',
            name='responsible_organization',
        ),
        migrations.AddField(
            model_name='organization',
            name='internal_type',
            field=models.CharField(choices=[('normal', 'Normal organization'), ('affiliated', 'Affiliated organization')], default='normal', max_length=20),
        ),
    ]
